// Require the 'net' module.
const net = require('net');
const LobbSys = require('./lobby');
const logsole = require('./logsole');

// Lobby Data
const lobby = {};
let lobbyId = 0;
let waiting = [];

// Create a net server.
net.createServer((socket) => {
  waiting.push(socket);

  // Check if the waiting list is full.
  if (waiting.length >= 3) {
    lobby[lobbyId] = new LobbSys(lobbyId);
  
    // Create a lobby for the players.
    waiting.forEach((player) => {
      lobby[lobbyId].insertPlayer(player);
    });

    // Start the game up.
    lobby[lobbyId].initialize();
    lobbyId += 1;
    waiting = [];
  }

  socket.on('data', (data) => {
    if (typeof socket.lobbyId === 'undefined') { return; }
    if (typeof lobby[socket.lobbyId] === 'undefined') { return; }
    const message = String(data).split('');
    for (let i = 0; i < message.length; i += 1) {
      lobby[socket.lobbyId].queueMovement(socket, message[i]);
    }
  });
-
  socket.on('error', () => {
    console.log('error');
    if (waiting.includes(socket)) { waiting.splice(socket, 1); }
    if (typeof socket.lobbyId === 'number' && lobby[socket.lobbyId].players.includes(socket)) {
      const lLobby = lobby[socket.lobbyId];
      lLobby.players[socket].killed = true;
      if (iLobby.players[socket].role === 'killer') { iLobby.dc = true; }
      if (iLobby.checkWinningConditions() === false) {
        lLobby.renderMaps();
      }
      socket.destroy();
    }
  });

  socket.on('close', () => {
    console.log('close');
    if (waiting.includes(socket)) { waiting.splice(socket, 1); }
    if (typeof socket.lobbyId === 'number' && lobby[socket.lobbyId].players.includes(socket)) {
      const lLobby = lobby[socket.lobbyId];
      lLobby.players[socket].killed = true;
      if (iLobby.players[socket].role === 'killer') { iLobby.dc = true; }
      if (iLobby.checkWinningConditions() === false) {
        lLobby.renderMaps();
      }
      socket.destroy();
    }
  });
}).listen(7777, '0.0.0.0', () => {
  console.clear();
  logsole.info('[tcp-pages-game] Server listening on 75.69.115.74:7777...');
  setInterval(() => {
    console.clear();
    logsole.info('[tcp-pages-game] Server listening on 75.69.115.74:7777...');
    Object.values(lobby).forEach((lby, indx) => {
      if (lby.over === true) {
        delete lobby[indx];
      } else if (lby.locked === true) {
        let players = 0;
        let playersTotal = 0;
        lby.players.forEach((player) => { if (player.killed === false && player.role === 'survivor') { players += 1; } });
        lby.players.forEach((player) => { if (player.role === 'survivor') { playersTotal += 1; } });
        logsole.log(`${indx === 0 ? '\n' : ''}Lobby ${lby.id} { ${players}/${playersTotal} Survivors; ${lby.map.pages.length} Pages Left; }`);
      }
    });
    waiting.forEach((player, indx) => {
      logsole.log(`${indx === 0 ? '\n' : ''}Waiting { ${player.remoteAddress}:${player.remotePort}; }`);
    });
  }, 1000);
});
